# -*- coding: iso-8859-15 -*-

# Contractor, a contract validation tool
#    Copyright (C) 2009  Guido de Caso, Alexis Tcach, Víctor Braberman, Diego Garbervetsky, Sebastián Uchitel
#    {gdecaso, atcach, vbraber, diegog, suchitel}@dc.uba.ar
#    LaFHIS - Departamento de Computación - Facultad de Ciencias Exactas y Naturales - Universidad de Buenos Aires
#    Pabellón I - Ciudad Universitaria - (C1428EGA) - Buenos Aires - Argentina
#    Tel: +54-11-4576-3390 eext. 704
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from input.options.options import Options

from output.writer import Writer
class LTSAWriter(Writer):

	def print_contents(self, fsm):
	
		# write header
		print "// AUTOGENERATED WITH CONTRACTOR (http://lafhis.dc.uba.ar/contractor)"
		print ""
	
		# write optional information header
		if Options.options.print_information:
			print "// EXTRA MODEL INFORMATION"
			from output.meta_information import MetaInformation
			for (attribute, value) in MetaInformation.info_attributes(fsm):
				print "// " + str(attribute) + ": " + str(value)
			print
	
		# write fsm name and link to initial state (if it exists)
		print fsm.name + " =",
		if fsm.initial_state == None:
			print "STOP."
		else:
			print fsm.initial_state + ","
		
			# write states
			states_size = len(fsm.states)
			for s in range(states_size):
				print fsm.states[s].name + " =", 
			
				state_remarks = self.get_state_remarks(fsm.states[s])
				if state_remarks != []:
					print "/* " + ", ".join(state_remarks) + " */"
					print "\t",
			
				# write transitions for this state
				transitions_size = len(fsm.states[s].transitions)
				if transitions_size == 0:
					print "STOP",
				else:
					print "(",
					for t in range(transitions_size):
						if t > 0:
							print "\t|",
						print uncapitalize(fsm.states[s].transitions[t].label) + " -> " + fsm.states[s].transitions[t].destination,
					
						transition_remarks = self.get_transition_remarks(fsm.states[s].transitions[t])
						if transition_remarks != []:
							print "\t/* " + ", ".join(transition_remarks) + " */",

						if t < transitions_size - 1:
							print ""
					print ")",
		
				# comma or dot?
				if s < states_size - 1:
					print ","
				else:
					print "."
		
	def get_state_remarks(self, state):
		ret = []
		if state.uncertain:
			ret.append("may be an inconsistent state")
		return ret
		
	def get_transition_remarks(self, transition):
		ret = []
		if transition.uncertain:
			ret.append("transition may not exist")
		if transition.violates_invariant:
			ret.append("may violate contract invariant on destination")
		return ret

# aux: uncapitalize a string (first character is lower-cased)
def uncapitalize(string):
	return string[0].lower() + string[1:]			
